<?php
// $Id: stafflist.admin.inc,v 1.1 2009/04/08 13:23:43 mwacker Exp $

define('STAFFLIST_DEPARTMENT', 0);
define('STAFFLIST_LEVEL', 1);

/**
 * Generates a list of departments or levels in weighted order.
 */
function _stafflist_role_list($type) {
  $result = db_query("SELECT s.rid, r.name, s.weight FROM {staff_roles} s, {role} r WHERE s.rid = r.rid AND s.type = %d ORDER BY s.weight ASC", $type);
  $list = array();
  while ($row = db_fetch_object($result)) {
    $list[$row->rid] = array('name' => $row->name, 'weight' => $row->weight);
  }
  return $list;
}

/**
 * Collects all the information for the staff list and passes it off for theming.
 */
function _stafflist_view() {
  // Initializes an empty list of users.
  $departments = _stafflist_role_list(STAFFLIST_DEPARTMENT);
  $levels = _stafflist_role_list(STAFFLIST_LEVEL);
  $users = array();
  foreach ($departments as $sdid => $v) {
    $users[$sdid] = array();
    foreach($levels as $slid => $v) {
      $users[$sdid][$slid] = array();
    }
  }
  // Fills up the list and passes it off to theme.
  $subquery = "SELECT ur.uid, ur.rid FROM {users_roles} ur, {staff_roles} s WHERE ur.rid = s.rid AND s.type = %d";
  $result = db_query("SELECT u.uid, sd.rid AS sdid, sr.rid AS srid FROM {users} u, ($subquery) sd, ($subquery) sr WHERE u.uid = sd.uid AND u.uid = sr.uid ORDER BY u.name ASC", STAFFLIST_DEPARTMENT, STAFFLIST_LEVEL);
  while ($row = db_fetch_object($result)){
    $users[$row->sdid][$row->srid][$row->uid] = $row->uid;
  }
  $page = theme('stafflist', $users, $departments, $levels);
  return $page;
}

/**
 * Default theme to display the staff list.
 */
function theme_stafflist($users, $departments, $levels) {
  $content = "";
  foreach ($users as $sdid => $dept_group) {
    $content .= "<h3>" . check_plain($departments[$sdid]['name']) . "</h3><ul>";
    foreach ($dept_group as $srid => $level_group) {
      $content .= "<li>" . check_plain($levels[$srid]['name']);
      if (count($level_group)) {
        $content .= "<ul>";
        foreach ($level_group as $uid) {
          $content .= "<li>". theme('username', user_load($uid)) ."</li>";
        }
        $content .= "</ul>";
      }
      else {
        $content .= "<ul><li><em>(None)</em></li></ul>";
      }
      $conent .= "</li>";
    }
    $content .= "</ul><hr />";
  }
  return $content;	
}

/**
 * Generates form to associates user roles with departments and levels.
 */
function stafflist_admin_roles() {
  $roles = user_roles(TRUE);
  $departments = _stafflist_role_list(STAFFLIST_DEPARTMENT);
  $levels = _stafflist_role_list(STAFFLIST_LEVEL);
  
  $form = array();
  $form['#role_list'] = array_keys($roles);
  $options = array(-1 => 'None', STAFFLIST_DEPARTMENT => 'Department', STAFFLIST_LEVEL => 'Level');
  
  foreach ($roles as $rid => $name) {
    if (array_key_exists($rid, $departments)) {
      $type = STAFFLIST_DEPARTMENT;
      $weight = $departments[$rid]['weight'];
    }
    elseif (array_key_exists($rid, $levels)) {
      $type = STAFFLIST_LEVEL;
      $weight = $levels[$rid]['weight'];
    }
    else {
      $type = -1;
      $weight = 0;
    }
    
    $form[$rid] = array(
      '#type' => 'fieldset',
      '#title' => $name,
    );
    $form[$rid][$rid .'_type'] = array(
      '#type' => 'radios',
      '#title' => 'Type',
      '#options' => $options,
      '#default_value' => $type,
    );
    $form[$rid][$rid .'_weight'] = array(
      '#type' => 'weight',
      '#title' => 'Weight',
      '#default_value' => $weight,
    );
  }
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Submit',
  );
  return $form;
}

/**
 * Processes form to associates user roles with departments and levels.
 */
function stafflist_admin_roles_submit($form, &$form_state) {
  $values = $form_state['values'];
  $args = array();
  $num_args = 0;
  foreach ($form['#role_list'] as $rid) {
    if ($values[$rid . '_type'] != -1) {
      $args = array_merge($args, array($rid, $values[$rid .'_type'], $values[$rid .'_weight']));
      $num_args++;
    }
  }
  db_query("DELETE FROM {staff_roles}");
  if ($num_args) {
    $query = "INSERT INTO {staff_roles}(rid, type, weight) VALUES ";
    $query .= implode(', ', array_fill(0, $num_args, '(%d, %d, %d)'));
    db_query($query, $args);
  }
}
